@@ -2,12 +2,12 @@
 /*                                                                           */
 /* This file is part of the SYMPHONY MILP Solver Framework.                  */
 /*                                                                           */
-/* SYMPHONY was jointly developed by Ted Ralphs (tkralphs@lehigh.edu) and    */
+/* SYMPHONY was jointly developed by Ted Ralphs (ted@lehigh.edu) and         */
 /* Laci Ladanyi (ladanyi@us.ibm.com).                                        */
 /*                                                                           */
-/* (c) Copyright 2000-2007 Ted Ralphs. All Rights Reserved.                  */
+/* (c) Copyright 2000-2015 Ted Ralphs. All Rights Reserved.                  */
 /*                                                                           */
-/* This software is licensed under the Common Public License. Please see     */
+/* This software is licensed under the Eclipse Public License. Please see    */
 /* accompanying file for terms.                                              */
 /*                                                                           */
 /*===========================================================================*/
@@ -15,9 +15,6 @@
 #include <stdlib.h>          /* malloc() is defined here in AIX ... */
 #include <stdio.h>
 #include <string.h>
-#ifdef _OPENMP
-#include "omp.h"
-#endif
 
 #include "sym_qsort.h"
 #include "sym_messages.h"
@@ -28,6 +25,7 @@
 #include "sym_master.h"
 #include "sym_master_u.h"
 #include "sym_lp_solver.h"
+#include "sym_primal_heuristics.h"
 #ifdef COMPILE_IN_TM
 #include "sym_lp.h"
 #endif
@@ -169,14 +167,14 @@ int io_u(sym_environment *env)
       }
       if (strcmp(env->par.datafile, "") == 0){
 	 if (env->par.file_type == LP_FORMAT){
-	    err = read_lp(env->mip, env->par.infile, env->probname);
-	    env->par.file_type = 0;
+	    err = read_lp(env->mip, env->par.infile, env->probname, env->par.verbosity);
+	    env->par.file_type = MPS_FORMAT;
 	    if (err != 0){
 	       printf("\nErrors in reading LP file\n");
 	       return (ERROR__READING_LP_FILE);
 	    }
 	 }else {
-	    err = read_mps(env->mip, env->par.infile, env->probname);
+	    err = read_mps(env->mip, env->par.infile, env->probname, env->par.verbosity);
 	    if (err != 0){
 	       printf("\nErrors in reading mps file\n");
 	       return (ERROR__READING_MPS_FILE);
@@ -186,15 +184,16 @@ int io_u(sym_environment *env)
 #ifdef USE_GLPMPL
 	 err = read_gmpl(env->mip, env->par.infile, 
 			 env->par.datafile, env->probname);
-	 env->par.file_type = 0;
+	 env->par.file_type = MPS_FORMAT;
 	 if(!err){
 	    printf("\nErrors in reading gmpl file\n");
 	    return (ERROR__READING_GMPL_FILE);
 	 }
 #else
 	 printf("ERROR: SYMPHONY can only read GMPL/AMPL files if GLPK is \n");
-	 printf("installed and the USE_GLMPL compiler define is set. \n");
+	 printf("installed and the USE_GLPMPL compiler define is set. \n");
 	 printf("Exiting.\n\n");
+	 return (ERROR__READING_GMPL_FILE);
 #endif
       }
       
@@ -363,11 +362,13 @@ int initialize_root_node_u(sym_environment *env)
 	 return(FUNCTION_TERMINATED_ABNORMALLY);
       }
 #endif
-      root->uind.list = (int *) malloc(root->uind.size * ISIZE);
-      for (i = 0; i < root->uind.size; i++){
-	 root->uind.list[i] = i;
+      if(root->uind.size){
+	 root->uind.list = (int *) malloc(root->uind.size * ISIZE);
+	 for (i = 0; i < root->uind.size; i++){
+	    root->uind.list[i] = i;
+	 }
       }
-      
+
       base->varnum = 0;
       base->userind = NULL;
 
@@ -456,12 +457,6 @@ int send_lp_data_u(sym_environment *env, int sender)
    int i;
    tm_prob *tm = env->tm;
    tm->par.max_active_nodes = env->par.tm_par.max_active_nodes;
-#ifdef _OPENMP
-   omp_set_dynamic(FALSE);
-   omp_set_num_threads(tm->par.max_active_nodes);
-#else
-   tm->par.max_active_nodes = 1;
-#endif
 
    tm->lpp = (lp_prob **) malloc(tm->par.max_active_nodes * sizeof(lp_prob *));
 
@@ -473,9 +468,8 @@ int send_lp_data_u(sym_environment *env, int sender)
 
       if ((tm->lpp[i]->has_ub = env->has_ub)){
 	 tm->lpp[i]->ub = env->ub;
-      }else{
-	 env->ub = - (MAXDOUBLE / 2);
       }
+      
       if (env->par.multi_criteria){
 	 if ((tm->lpp[i]->has_mc_ub = env->has_mc_ub)){
 	    tm->lpp[i]->mc_ub = env->mc_ub;
@@ -503,11 +497,11 @@ int send_lp_data_u(sym_environment *env, int sender)
 
    s_bufid = init_send(DataInPlace);
    send_char_array((char *)(&env->par.lp_par), sizeof(lp_params));
-   send_char_array(&env->has_ub, 1);
+   send_int_array(&env->has_ub, 1);
    if (env->has_ub)
       send_dbl_array(&env->ub, 1);
    if (env->par.multi_criteria){
-      send_char_array(&env->has_mc_ub, 1);
+      send_int_array(&env->has_mc_ub, 1);
       if (env->has_mc_ub){
 	 send_dbl_array(&env->mc_ub, 1);
 	 send_dbl_array(env->obj, 2);
@@ -549,7 +543,7 @@ int send_lp_data_u(sym_environment *env, int sender)
 	 has_colnames = TRUE;
 	 send_char_array(&has_colnames, 1);
 	 for (i = 0; i < mip->n; i++){
-	    send_char_array(mip->colname[i], 8);
+	    send_char_array(mip->colname[i], MAX_NAME_SIZE);
 	 }
       }else{
 	 send_char_array(&has_colnames, 1);
@@ -641,12 +635,24 @@ int send_cp_data_u(sym_environment *env, int sender)
 
 int display_solution_u(sym_environment *env, int thread_num)
 {
-   int user_res, i;
+   int user_res, i, n; 
    lp_sol sol;
-
+   char **colname;
+   
+   memset(&sol, 0, sizeof(lp_sol));
+   
    sol.xlength = 0;
 
-   if (env->par.verbosity < 0){
+
+   if (env->orig_mip) {
+      colname = env->orig_mip->colname;
+      n = env->orig_mip->n; 
+   } else {
+      colname = env->mip->colname;
+      n = env->mip->n; 
+   }
+   
+   if (env->par.verbosity < -1){
        return(FUNCTION_TERMINATED_NORMALLY);
    }     
    
@@ -664,6 +670,9 @@ int display_solution_u(sym_environment *env, int thread_num)
 
    if (!sol.has_sol){
       switch(env->termcode){
+       case TM_UNBOUNDED:
+	 printf("\nThe problem is unbounded!\n\n");
+	 return(FUNCTION_TERMINATED_NORMALLY);
        case TM_NO_SOLUTION:
 	  printf("\nThe problem is infeasible!");
 	  break;
@@ -676,10 +685,10 @@ int display_solution_u(sym_environment *env, int thread_num)
    
    printf("\nSolution Found: Node %i, Level %i\n", sol.xindex, sol.xlevel);
    if (env->par.multi_criteria){
-      printf("First Objective: %.3f\n", env->obj[0]);
-      printf("Second Objective: %.3f\n", env->obj[1]);
+      printf("First Objective: %.10f\n", env->obj[0]);
+      printf("Second Objective: %.10f\n", env->obj[1]);
    }else{
-      printf("Solution Cost: %.3f\n", env->mip->obj_sense == SYM_MINIMIZE ? 
+      printf("Solution Cost: %.10f\n", env->mip->obj_sense == SYM_MINIMIZE ? 
 	     sol.objval + env->mip->obj_offset : 
 	     -sol.objval + env->mip->obj_offset);
    }
@@ -693,50 +702,62 @@ int display_solution_u(sym_environment *env, int thread_num)
    user_res = USER_DEFAULT;
 #endif
 
-   switch(user_res){
-    case USER_SUCCESS:
-      return(FUNCTION_TERMINATED_NORMALLY);
-    case USER_DEFAULT:
-      if (sol.xlength){
-	 if (env->mip->colname){ 
-	    printf("+++++++++++++++++++++++++++++++++++++++++++++++++++\n");
-	    printf("Column names and values of nonzeros in the solution\n");
-	    printf("+++++++++++++++++++++++++++++++++++++++++++++++++++\n");
-	    for (i = 0; i < sol.xlength; i++){
-	       if (sol.xind[i] == env->mip->n){
-		  continue;
-	       }
-	       printf("%8s %10.3f\n", env->mip->colname[sol.xind[i]],
-		      sol.xval[i]);
-	    }
-	    printf("\n");
-	 }else{
-	    printf("+++++++++++++++++++++++++++++++++++++++++++++++++++\n");
-	    printf("User indices and values of nonzeros in the solution\n");
-	    printf("+++++++++++++++++++++++++++++++++++++++++++++++++++\n");
-	    for (i = 0; i < sol.xlength; i++){
-	       if (sol.xind[i] == env->mip->n){
-		  continue;
-	       }
-	       printf("%7d %10.3f\n", sol.xind[i], sol.xval[i]);
-	    }
-	    printf("\n");
-	 }
-	 
-	 return(FUNCTION_TERMINATED_NORMALLY);
-      }else{
-	 printf("+++++++++++++++++++++++++++++++++++++++++++++++++++\n");
-	 printf("All columns are zero in the solution!\n");
-	 printf("+++++++++++++++++++++++++++++++++++++++++++++++++++\n");
-	 return(FUNCTION_TERMINATED_NORMALLY);
-      }
-    case USER_ERROR:
-      return(FUNCTION_TERMINATED_NORMALLY);
-      
-    default:
-      return(FUNCTION_TERMINATED_NORMALLY);
-   }
+   if (env->par.verbosity > -1){
+      switch(user_res){
+       case USER_SUCCESS:
+	  return(FUNCTION_TERMINATED_NORMALLY);
+       case USER_DEFAULT:
+	  if (sol.xlength){
+	     if (env->mip->colname){ 
+		printf("+++++++++++++++++++++++++++++++++++++++++++++++++++");
+		printf("\n");
+		printf("Column names and values of nonzeros in the solution");
+		printf("\n");
+		printf("+++++++++++++++++++++++++++++++++++++++++++++++++++");
+		printf("\n");
 
+		for (i = 0; i < sol.xlength; i++){
+		   if (sol.xind[i] >= n){
+		      continue;
+		   }
+		   printf("%-50s %10.10f\n", colname[sol.xind[i]],
+			  sol.xval[i]);
+		}
+		
+		printf("\n");
+	     }else{
+		printf("+++++++++++++++++++++++++++++++++++++++++++++++++++");
+		printf("\n");
+		printf("User indices and values of nonzeros in the solution");
+		printf("\n");
+		printf("+++++++++++++++++++++++++++++++++++++++++++++++++++");
+		printf("\n");
+
+		for (i = 0; i < sol.xlength; i++){
+		   if (sol.xind[i] >= n){
+		      continue;
+		   }
+		   printf("%7d %10.10f\n", sol.xind[i], sol.xval[i]);
+		}
+		
+		printf("\n");
+	     }
+	     
+	     return(FUNCTION_TERMINATED_NORMALLY);
+	  }else{
+	     printf("+++++++++++++++++++++++++++++++++++++++++++++++++++\n");
+	     printf("All columns are zero in the solution!\n");
+	     printf("+++++++++++++++++++++++++++++++++++++++++++++++++++\n");
+	     return(FUNCTION_TERMINATED_NORMALLY);
+	  }
+       case USER_ERROR:
+	  return(FUNCTION_TERMINATED_NORMALLY);
+	  
+       default:
+	  return(FUNCTION_TERMINATED_NORMALLY);
+      }
+   }
+   
    return(FUNCTION_TERMINATED_NORMALLY);
 }
 
@@ -766,6 +787,11 @@ int free_master_u(sym_environment *env)
       free_mip_desc(env->mip);
       FREE(env->mip);
    }
+
+   if(env->prep_mip){
+      free_mip_desc(env->prep_mip);
+      FREE(env->prep_mip);
+   }
    
    if (env->rootdesc){
       FREE(env->rootdesc->desc);
@@ -783,6 +809,10 @@ int free_master_u(sym_environment *env)
 #ifdef COMPILE_IN_TM
    if (env->warm_start){
       free_subtree(env->warm_start->rootnode);
+      if(env->warm_start->best_sol.has_sol){
+	 FREE(env->warm_start->best_sol.xind);
+	 FREE(env->warm_start->best_sol.xval);
+      }
       if (env->warm_start->cuts){
 	 for (i = env->warm_start->cut_num - 1; i >= 0; i--){
 	    if (env->warm_start->cuts[i]){
@@ -805,7 +835,11 @@ int free_master_u(sym_environment *env)
    }
 #endif
 #endif
-      
+   if (env->sp){
+      sp_free_sp(env->sp);
+      FREE(env->sp);
+   }
+   
    return(FUNCTION_TERMINATED_NORMALLY);   
 }
 
